package cz.tendersystems.helpdesk.service.organizationalstructure;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.general.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class PasswordService {

	private static final Integer automaticallyGeneratePasswordLenght = 16;
	private static final String linkForChangePassword = "/changePassword/setPassword.jsf";
	private static final Integer keyLenght = 50;
	private static final Integer defaultForceLinkForChangePassword = 7;
	private static final String templateNameForFirstConfigurationPassword = "linkForFirstConfigurationPassword";
	private static final String templateNameForChangePassword = "linkForChangePassword";

	@PersistenceContext
	EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	MailService mailService;

	@EJB
	ConfigurationService configurationService;

	@EJB
	ApplicationLogService applicationLogService;

	public void changePassword(String key, String newPassword) throws ValidationException {
		SystemUser user = findUser(key);

		ValidationPassword.validatePassword(newPassword, user);

		user.setPassword(Tools.generateHash(newPassword, user.getLoginName()));
		user.setChangePassword(new ChangePassword());

		createLog(new ApplicationLogData("Změna hesla", "Změna hesla uživatele"), user.getLoginName());
	}

	public void sendLinkForChangePassword(String loginName) throws ValidationException {
		SystemUser user = em.find(SystemUser.class, loginName);

		if (user == null || user.getUserAccountStatus().equals(UserAccountStatus.FORBIDDEN)) {
			throw new ValidationException("error.userNotExistOrIsFormidden", loginName);
		}

		user.setChangePassword(prepareChangePassword());
		sendLink(templateNameForChangePassword, user, prepareLinkForChangePassword(user.getChangePassword().getKey()), user.getChangePassword().getExpirationDate());
	}

	public void sendLinkForFirstConfigurationPassword(SystemUser user) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni odeslat odkaz pro prvotni nastaveni hesla");
		}

		user.setChangePassword(prepareChangePassword());
		sendLink(templateNameForFirstConfigurationPassword, user, prepareLinkForChangePassword(user.getChangePassword().getKey()), user.getChangePassword().getExpirationDate());
	}

	public boolean checkTheValidityOfKey(String key) {
		SystemUser user = findUser(key);

		return ValidationPassword.isValidKey(user);
	}

	public String preparePasswordHash(String loginName) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni pripravit hash hesla");
		}

		if (loginName == null) {
			throw new HelpdeskException("Prihlasovaci jmeno nemuze byt null");
		}

		return Tools.generateHash(Tools.generateRandomString(automaticallyGeneratePasswordLenght), loginName);
	}

	private ChangePassword prepareChangePassword() {
		return new ChangePassword(Tools.generateRandomString(keyLenght), prepareExpirationDateLink());
	}

	private Date prepareExpirationDateLink() {
		return moveDate(defaultForceLinkForChangePassword, Calendar.DAY_OF_YEAR);
	}

	private String prepareLinkForChangePassword(String key) {
		Map<String, String> parameters = new HashMap<>();
		parameters.put("key", key);

		return Tools.generateLink(configurationService.getApplicationDomain(), linkForChangePassword, parameters);
	}

	private void sendLink(String templateName, SystemUser recipient, String link, Date expirationDate) {
		mailService.sendMail(templateName, recipient.getPerson().getMail(), recipient.getLoginName(), link, Tools.provideDateWithTime(expirationDate));
	}

	private SystemUser findUser(String key) {
		try {
			return (SystemUser) em.createQuery("SELECT u FROM SystemUser u WHERE u.changePassword.key = :key").setParameter("key", key).getSingleResult();
		} catch (NoResultException e) {
			return null;
		}
	}

	private Date moveDate(Integer number, Integer unitOfTime) {
		Calendar calendar = Calendar.getInstance();
		calendar.add(unitOfTime, number);

		return calendar.getTime();
	}

	private void createLog(ApplicationLogData applicationLog, String loginName) {
		applicationLog.setLoginName(loginName);
		applicationLogService.createLog(SystemUser.class, applicationLog);
	}
}
