package cz.tendersystems.helpdesk.service.organizationalstructure;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class UserService {

	@PersistenceContext
	EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	OutsourcerService outsourcerService;

	@EJB
	ProductService productService;

	@EJB
	PasswordService passwordService;

	@EJB
	ApplicationLogService applicationLogService;

	public SystemUser findUser(String loginName) {
		SystemUser user = em.find(SystemUser.class, loginName);

		if (user == null) {
			throw new HelpdeskException("Uzivatel s prihlasovacim jmenem {0} neexistuje", loginName);
		}

		return user;
	}

	public SystemUserData loadUser(String loginName) {
		SystemUser user = Tools.prepareEntityOfRightType(findUser(loginName));

		return prepareEntityOfRightType(user);
	}

	private SystemUserData prepareEntityOfRightType(SystemUser user) {
		if (user instanceof UserSolver) {
			return new UserSolverData((UserSolver) user);
		} else if (user instanceof UserOutsourcer) {
			return new UserOutsourcerData((UserOutsourcer) user);
		}

		throw new HelpdeskException("Objekt neodpovida zadne instanci tridy");
	}

	public String createUser(SystemUserData userData) throws ValidationException {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni vytvorit uzivatele");
		}

		validateUser(userData, true);

		SystemUser user = prepareCreateUser(userData);
		user.setLoginName(userData.getLoginName());

		user.setPassword(passwordService.preparePasswordHash(userData.getLoginName()));

		em.persist(user);

		applicationLogService.createLog(SystemUser.class, "Vytvoření uživatele", "Vytvoření uživatele s přihlašovacím jménem {0}", user.getLoginName());

		passwordService.sendLinkForFirstConfigurationPassword(user);

		return userData.getLoginName();
	}

	private SystemUser prepareCreateUser(SystemUserData userData) {
		if (userData instanceof UserSolverData) {
			UserSolver userSolver = new UserSolver((UserSolverData) userData);
			userSolver.setProducts(new ArrayList<>());
			((UserSolverData) userData).getProducts().forEach(p -> userSolver.getProducts().add(productService.findProduct(p.getId())));

			return userSolver;
		} else if (userData instanceof UserOutsourcerData) {
			UserOutsourcer userOutsourcer = new UserOutsourcer((UserOutsourcerData) userData);
			userOutsourcer.setOutsourcer(outsourcerService.findOutsourcer(((UserOutsourcerData) userData).getOutsourcer().getId()));

			return userOutsourcer;
		}

		throw new HelpdeskException("Objekt {0} neodpovida zadne instanci tridy Uzivatel.", userData.getClass());
	}

	public void updateUser(SystemUserData userData) throws ValidationException {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit uzivatele");
		}

		validateUser(userData, false);

		if (userData instanceof UserSolverData) {
			prepareUpdateUserSolver((UserSolverData) userData);
		} else if (userData instanceof UserOutsourcerData) {
			prepareUpdateUserOutsourcer((UserOutsourcerData) userData);
		}

		applicationLogService.createLog(SystemUser.class, "Upravení uživatele", "Upravení uživatele s přihlašovacím jménem {0}", userData.getLoginName());
	}

	public void changeUserAcountStatus(String loginName, UserAccountStatus userAccountStatus) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni zmenit stav uzivatele");
		}

		findUser(loginName).setUserAccountStatus(userAccountStatus);

		applicationLogService.createLog(SystemUser.class, "Změna stavu uživatele", "Změna stavu uživatele s přihlašovacím jménem {0}", loginName);
	}

	@SuppressWarnings("unchecked")
	public List<UserSolverData> loadUserOfProduct(Long idProduct) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni pripravit uzivatele produktu");
		}

		String HQL = "SELECT u FROM UserSolver u LEFT JOIN FETCH u.products p WHERE p.id = :idProduct";

		List<UserSolver> users = em.createQuery(HQL).setParameter("idProduct", idProduct).getResultList();

		List<UserSolverData> data = new ArrayList<>();

		users.forEach(u -> data.add(new UserSolverData(u)));

		return data;
	}

	@SuppressWarnings("unchecked")
	public List<UserOutsourcerData> loadUserOfOutsourcer(Long idOutsourcer) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni pripravit uzivatele zadavatele");
		}

		String HQL = "SELECT u FROM UserOutsourcer u LEFT JOIN FETCH u.outsourcer o WHERE o.id = :idOutsourcer";

		List<UserOutsourcer> users = em.createQuery(HQL).setParameter("idOutsourcer", idOutsourcer).getResultList();

		List<UserOutsourcerData> data = new ArrayList<>();

		users.forEach(u -> data.add(new UserOutsourcerData(u)));

		return data;
	}

	private void validateUser(SystemUserData user, boolean isCreate) throws ValidationException {
		if (!isLoginNameAvailable(user.getLoginName()) && isCreate) {
			throw new ValidationException("error.loginNameIsNotAvailable");
		}

		ValidationUser.validateUser(user);
	}

	private void prepareUpdateUserSolver(UserSolverData data) {
		List<Product> newProducts = new ArrayList<>();
		data.getProducts().forEach(p -> newProducts.add(productService.findProduct(p.getId())));

		UserSolver user = (UserSolver) findUser(data.getLoginName());
		List<Product> oldProducts = user.getProducts();

		UserSolver.convert(user, data);

		user.getProducts().removeAll(oldProducts);
		user.setProducts(newProducts);
	}

	private void prepareUpdateUserOutsourcer(UserOutsourcerData data) {
		UserOutsourcer user = (UserOutsourcer) findUser(data.getLoginName());
		UserOutsourcer.convert(user, data);
		user.setOutsourcer(outsourcerService.findOutsourcer(data.getOutsourcer().getId()));
	}

	private boolean isLoginNameAvailable(String loginName) {
		return (boolean) em.createQuery("SELECT CASE WHEN COUNT(u) > 0 THEN false ELSE true END FROM SystemUser u WHERE u.loginName = :loginName").setParameter("loginName", loginName).getSingleResult();
	}
}
