package cz.tendersystems.helpdesk.service.organizationalstructure.sla;

import java.text.*;
import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class SlaSchemaService {

	@PersistenceContext
	private EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	ApplicationLogService applicationLogService;

	@EJB
	TimeActivityService timeActivityService;

	@EJB
	ProductService productService;

	@EJB
	OutsourcerService outsourcerService;

	@EJB
	UserService userService;

	public SlaSchema findSlaSchema(Long idSlaSchema) {
		SlaSchema slaSchema = em.find(SlaSchema.class, idSlaSchema);

		if (slaSchema == null) {
			throw new HelpdeskException("Sla schema {0} neexistuje", idSlaSchema);
		}

		return slaSchema;
	}

	public SlaSchemaData loadSlaSchema(Long idSlaSchema) {
		return new SlaSchemaData(findSlaSchema(idSlaSchema));
	}

	private Assignment findAssignment(Long idAssignment) {
		final String HQL = "SELECT a FROM Assignment a LEFT JOIN FETCH a.product LEFT JOIN FETCH a.slaSchema LEFT JOIN FETCH a.outsourcer WHERE a.id = :idAssignment";

		return (Assignment) em.createQuery(HQL).setParameter("idAssignment", idAssignment).getSingleResult();
	}

	public AssignmentData loadAssignmentSlaSchema(Long idSlaSchema) {
		String HQL = "SELECT a FROM Assignment a LEFT JOIN FETCH a.product LEFT JOIN FETCH a.slaSchema s LEFT JOIN FETCH a.outsourcer WHERE s.id = :idSlaSchema";
		Assignment assignment = (Assignment) em.createQuery(HQL).setParameter("idSlaSchema", idSlaSchema).getSingleResult();

		return new AssignmentData(assignment);
	}

	public Long createSlaSchema(AssignmentData data, List<TimeActivityData> timesActivity) throws ValidationException {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni vytvorit sla schema");
		}

		ValidationSlaSchema.validationSlaSchema(data, timesActivity);

		Assignment asignment = new Assignment(data);
		prepareAssignmentBeforeSave(asignment, data);

		em.persist(asignment);

		timeActivityService.updateTimesActivity(timesActivity, asignment.getSlaSchema());

		applicationLogService.createLog(SlaSchema.class, "Vytvoření sla schéma", "Vytvoření sla schéma s id {0}", asignment.getSlaSchema().getId());

		return asignment.getSlaSchema().getId();
	}

	public void updateSlaSchema(AssignmentData data, List<TimeActivityData> timesActivity) throws ValidationException {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit sla schema");
		}

		ValidationSlaSchema.validationSlaSchema(data, timesActivity);

		Assignment assignment = findAssignment(data.getId());
		SlaSchema slaSchema = findSlaSchema(data.getSlaSchema().getId());
		SlaSchema.convert(slaSchema, data.getSlaSchema());

		assignment.setSlaSchema(slaSchema);
		prepareAssignmentBeforeSave(assignment, data);

		em.persist(assignment);

		timeActivityService.updateTimesActivity(timesActivity, assignment.getSlaSchema());

		applicationLogService.createLog(SlaSchema.class, "Upravení sla schéma", "Upravení sla schéma s id {0}", data.getId());
	}

	public void updateActivityOfSlaSchema(SlaSchemaData data, boolean active) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit aktivitu sla schema");
		}

		findSlaSchema(data.getId()).setActive(active);

		applicationLogService.createLog(SlaSchema.class, "Změna aktivity sla schéma", "Změna aktivity sla schéma s id {0}", data.getId());
	}

	@SuppressWarnings("unchecked")
	public List<SlaSchemaData> loadSlaSchemaProduct(Long idProduct, AuthorData author) {
		String HQL = "SELECT NEW cz.tendersystems.helpdesk.data.organizationalstructure.sla.SlaSchemaData(s) FROM Assignment a LEFT JOIN a.product p LEFT JOIN a.slaSchema s LEFT JOIN a.outsourcer o WHERE p.id = :idProduct AND {0}";

		if (basicSecurityService.isRoleType(RoleType.OUTSOURCER)) {
			return em.createQuery(MessageFormat.format(HQL, "o.id = :idOutsourcer AND s.active IS TRUE")).setParameter("idProduct", idProduct).setParameter("idOutsourcer", ((UserOutsourcerData) userService.loadUser(basicSecurityService.getLoginName())).getOutsourcer().getId()).getResultList();
		}

		if (author != null) {
			if (author.getCompanyName() == null) {
				return em.createQuery(MessageFormat.format(HQL, "o.id = :idOutsourcer")).setParameter("idProduct", idProduct).setParameter("idOutsourcer", author.getLoggedUser().getOutsourcer().getId()).getResultList();
			} else {
				return em.createQuery(MessageFormat.format(HQL, "o IS NULL")).setParameter("idProduct", idProduct).getResultList();
			}
		}

		return em.createQuery(MessageFormat.format(HQL, "o IS NULL AND s.active IS TRUE")).setParameter("idProduct", idProduct).getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<SlaSchemaData> loadSlaSchemas() {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni nacist seznam sla schema");
		}

		String HQL = "SELECT NEW cz.tendersystems.helpdesk.data.organizationalstructure.sla.SlaSchemaData(MAX(s.id), s.name) FROM SlaSchema s GROUP BY s.name";

		return em.createQuery(HQL).getResultList();
	}

	private void prepareAssignmentBeforeSave(Assignment assignment, AssignmentData data) {
		assignment.setProduct(productService.findProduct(data.getProduct().getId()));
		assignment.setOutsourcer(data.getOutsourcer() == null ? null : outsourcerService.findOutsourcer(data.getOutsourcer().getId()));
	}
}
