package cz.tendersystems.helpdesk.service.request;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class CommentService {

	private static final String linkForDetailRequest = "/request/detail.jsf";
	private static final String templateNameForCreateRequest = "createRequest";
	private static final String templateNameForUpdateRequest = "updateRequest";

	@PersistenceContext
	EntityManager em;

	@EJB
	RequestSecurityService requestSecurityService;

	@EJB
	ApplicationLogService applicationLogService;

	@EJB
	RequestService requestService;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	UserService userService;

	@EJB
	MailService mailService;

	@EJB
	SlaSchemaService slaSchemaService;

	@EJB
	ConfigurationService configurationService;

	@EJB
	FileService fileService;

	@SuppressWarnings("unchecked")
	public List<CommentData> loadCommentRequest(String idRequest) {
		if (!requestSecurityService.authorityLoadRequest(idRequest)) {
			throw new SecurityException("Nedostatecne opravneni nacist komentare");
		}

		String HQL = "SELECT NEW cz.tendersystems.helpdesk.data.request.CommentData(c) FROM Comment c LEFT JOIN c.request r WHERE r.id = :idRequest ORDER BY c.id ASC";

		return em.createQuery(HQL).setParameter("idRequest", idRequest).getResultList();
	}

	public void createCommentAtCreateRequest(Request request, CommentData data, List<FileData> attachments) {
		Comment comment = new Comment(data);
		comment.setCreatedDateTime(new Date());
		comment.setAuthor(prepareAuthor(request.getAuthor()));

		comment.setRequest(request);

		em.persist(comment);
		request.setComments(Arrays.asList(comment));

		createLog(comment.getId());

		fileService.saveFiles(attachments, comment);

		sendLink(templateNameForCreateRequest, request);
	}

	public void createCommentAtSolve(Request request, CommentData data, List<FileData> attachments) {
		Comment comment = new Comment(data);
		comment.setCreatedDateTime(new Date());
		comment.setRequest(request);

		em.persist(comment);
		request.getComments().add(comment);

		createLog(comment.getId());

		fileService.saveFiles(attachments, comment);

		sendLink(templateNameForUpdateRequest, request);
	}

	public void createComment(String idRequest, CommentData data, List<FileData> attachments) {
		if (!requestSecurityService.authorityCreateComment(idRequest)) {
			throw new SecurityException("Nedostatecne opravneni vytvorit komentar");
		}

		Comment comment = new Comment(data);
		Request request = requestService.findRequest(idRequest);

		if (basicSecurityService.isUserLogged() && comment.getAuthor() == null) {
			comment.setAuthor(userService.loadUser(basicSecurityService.getLoginName()).getPerson().getMail());
		}

		comment.setRequestStatus(request.getRequestStatus());
		comment.setCreatedDateTime(new Date());
		comment.setRequest(request);

		em.persist(comment);

		createLog(comment.getId());

		fileService.saveFiles(attachments, comment);

		sendLink(templateNameForUpdateRequest, prepareRequest(comment));
	}

	private String prepareAuthor(Author author) {
		if (author.getAnonymous() != null) {
			return author.getAnonymous().getMail();
		} else {
			return author.getLoggedUser().getPerson().getMail();
		}
	}

	private void createLog(Long idComment) {
		applicationLogService.createLog(Comment.class, "Vytvoření komentáře", "Vytvoření komentáře s id {0}", idComment);
	}

	private void sendLink(String templateName, Request request) {
		mailService.sendMail(templateName, prepareRecipients(request.getComments()), slaSchemaService.loadAssignmentSlaSchema(request.getSlaSchema().getId()).getProduct().getName(), prepareLinkForDetailRequest(request.getId()));
	}

	private Request prepareRequest(Comment comment) {
		Request request = comment.getRequest();
		comment.getRequest().getComments().add(comment);

		return request;
	}

	private List<String> prepareRecipients(List<Comment> comments) {
		List<String> recipients = new ArrayList<>();

		for (Comment comment : comments) {
			if (!recipients.equals(comment.getAuthor())) {
				recipients.add(comment.getAuthor());
			}
		}

		return recipients;
	}

	private String prepareLinkForDetailRequest(String id) {
		Map<String, String> parameters = new HashMap<>();
		parameters.put("id", id);

		return Tools.generateLink(configurationService.getApplicationDomain(), linkForDetailRequest, parameters);
	}
}
