package cz.tendersystems.helpdesk.service.request;

import java.text.*;
import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.general.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class RequestService {

	@PersistenceContext
	EntityManager em;

	@EJB
	IdService idService;

	@EJB
	RequestSecurityService requestSecurityService;

	@EJB
	ApplicationLogService applicationLogService;

	@EJB
	UserService userService;

	@EJB
	CommentService commentService;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	SlaSchemaService slaSchemaService;

	public Request findRequest(String idRequest) {
		if (!requestSecurityService.authorityLoadRequest(idRequest)) {
			throw new SecurityException("Nedostatecne opravneni nacist pozadavek");
		}

		Request request = em.find(Request.class, idRequest);

		return request;
	}

	public RequestData loadRequest(String idRequest) {
		Request request = findRequest(idRequest);

		return new RequestData(request);
	}

	public String createRequest(RequestData data, CommentData comment, List<FileData> attachments) throws ValidationException {
		ValidationRequest.validateRequest(data, comment, data.getRequestStatus());

		Request request = new Request(data);

		request.setId(idService.createId(data));
		request.setCreatedDateTime(new Date());
		request.setAuthor(new Author(data.getAuthor()));
		request.setSlaSchema(slaSchemaService.findSlaSchema(data.getSlaSchema().getId()));

		prepareDateTime(request);

		if (request.getAuthor().getLoggedUser() != null) {
			request.getAuthor().setLoggedUser((UserOutsourcer) userService.findUser(data.getAuthor().getLoggedUser().getLoginName()));
		}

		em.persist(request);

		applicationLogService.createLog(Request.class, new ApplicationLogData(LevelRecord.INFORMATION, "Vytvoření požadavku", request.getId(), MessageFormat.format("Vytvoření požadavku s id {0}", request.getId())));

		commentService.createCommentAtCreateRequest(request, comment, attachments);

		return request.getId();
	}

	public void updateRequest(RequestData data, CommentData comment, RequestStatus status, List<FileData> attachments) throws ValidationException {
		if (!requestSecurityService.authorityUpdateRequest(data.getId(), status)) {
			throw new SecurityException("Nedostatecne opravneni upravit pozadavek");
		}

		ValidationRequest.validateRequest(data, comment, status);

		Request request = findRequest(data.getId());

		Request.convert(request, data);
		request.setRequestStatus(status);
		request.setSlaSchema(slaSchemaService.findSlaSchema(data.getSlaSchema().getId()));
		request.setAuthor(em.find(Author.class, data.getAuthor().getId()));
		request.setSolver((UserSolver) userService.findUser(basicSecurityService.getLoginName()));

		prepareDateTime(request);

		if (RequestStatus.SOLVED.equals(request.getRequestStatus())) {
			request.setDateOfClose(new Date());
		}

		comment.setAuthor(request.getSolver().getPerson().getMail());
		comment.setRequestStatus(status);

		applicationLogService.createLog(Request.class, new ApplicationLogData(LevelRecord.INFORMATION, "Změna požadavku", request.getId(), MessageFormat.format("Změna požadavku požadavku s id {0}", request.getId())));

		commentService.createCommentAtSolve(request, comment, attachments);
	}

	private void prepareDateTime(Request request) {
		ToolsForRequestDates toolsForRequestDates = new ToolsForRequestDates();
		toolsForRequestDates.calculateRequestDates(request);

		request.setReactionTime(toolsForRequestDates.getReactionTime());
		request.setTimeToSolve(toolsForRequestDates.getTimeToSolve());
	}
}
