package cz.tendersystems.helpdesk.service.request.list;

import java.util.*;
import javax.persistence.criteria.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.service.list.*;

public class RequestFilter implements Filter<Request> {

	private String id;
	private String name;
	private RequestStatus requestStatus;
	private ProductData product;
	private SlaSchemaData slaSchema;
	private String authorName;
	private String authorSurname;
	private String authorCompanyName;
	private String solver;
	private Date byCreatedDateTime;
	private Date toCreatedDateTime;
	private Date byReactionTime;
	private Date toReactionTime;
	private Date byTimeToSolve;
	private Date toTimeToSolve;
	private Date byDateOfClose;
	private Date toDateOfClose;

	public RequestFilter() {
	}

	@Override
	public List<Predicate> preparePredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		List<Predicate> predicates = new ArrayList<Predicate>();

		if (getId() != null) {
			predicates.add(builder.like(root.<String> get("id"), "%" + getId() + "%"));
		}

		if (getName() != null) {
			predicates.add(builder.like(root.<String> get("name"), "%" + getName() + "%"));
		}

		if (getRequestStatus() != null) {
			predicates.add(builder.equal(root.<Boolean> get("requestStatus"), getRequestStatus()));
		}

		if (getProduct() != null) {
			predicates.add(preparePredicateOnProduct(builder, query, root));
		}

		if (getSlaSchema() != null) {
			predicates.add(preparePredicateOnSlaSchema(builder, query, root));
		}

		if (getSolver() != null) {
			predicates.add(preparePredicateOnSolver(builder, query, root));
		}

		if (getAuthorName() != null || getAuthorSurname() != null || getAuthorCompanyName() != null) {
			predicates.add(builder.or(preparePredicateOnAnonymousAuthor(builder, query, root), preparePredicateOnLoggedAuthor(builder, query, root)));
		}

		if (getByCreatedDateTime() != null) {
			predicates.add(builder.greaterThanOrEqualTo(root.<Date> get("createdDateTime"), getByCreatedDateTime()));
		}

		if (getToCreatedDateTime() != null) {
			predicates.add(builder.lessThanOrEqualTo(root.<Date> get("createdDateTime"), getToCreatedDateTime()));
		}

		if (getByReactionTime() != null) {
			predicates.add(builder.greaterThanOrEqualTo(root.<Date> get("reactionTime"), getByReactionTime()));
		}

		if (getToReactionTime() != null) {
			predicates.add(builder.lessThanOrEqualTo(root.<Date> get("reactionTime"), getToReactionTime()));
		}

		if (getByTimeToSolve() != null) {
			predicates.add(builder.greaterThanOrEqualTo(root.<Date> get("timeToSolve"), getByTimeToSolve()));
		}

		if (getToTimeToSolve() != null) {
			predicates.add(builder.lessThanOrEqualTo(root.<Date> get("timeToSolve"), getToTimeToSolve()));
		}

		if (getByDateOfClose() != null) {
			predicates.add(builder.greaterThanOrEqualTo(root.<Date> get("dateOfClose"), getByDateOfClose()));
		}

		if (getToDateOfClose() != null) {
			predicates.add(builder.lessThanOrEqualTo(root.<Date> get("dateOfClose"), getToDateOfClose()));
		}

		return predicates;
	}

	private Predicate preparePredicateOnProduct(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<Assignment> rootSubquery = subquery.from(Assignment.class);

		subquery.select(rootSubquery.<Long> get("id"));
		subquery.where(builder.like(builder.lower(builder.function("str", String.class, rootSubquery.join("product").<Long> get("id"))), "%" + getProduct().getId() + "%"));

		return root.join("slaSchema").<Long> get("id").in(subquery);
	}

	private Predicate preparePredicateOnSlaSchema(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<SlaSchema> rootSubquery = subquery.from(SlaSchema.class);

		subquery.select(rootSubquery.<Long> get("id"));
		subquery.where(builder.like(rootSubquery.<String> get("name"), "%" + getSlaSchema().getName() + "%"));

		return root.join("slaSchema").<Long> get("id").in(subquery);
	}

	private Predicate preparePredicateOnSolver(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<String> subquery = query.subquery(String.class);
		Root<UserSolver> rootSubquery = subquery.from(UserSolver.class);

		subquery.select(rootSubquery.<String> get("loginName"));
		subquery.where(builder.like(rootSubquery.<String> get("loginName"), "%" + getSolver() + "%"));

		return root.join("solver").<String> get("loginName").in(subquery);
	}

	private Predicate preparePredicateOnAnonymousAuthor(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<Author> rootSubquery = subquery.from(Author.class);

		subquery.select(rootSubquery.<Long> get("id"));

		Predicate predicateOnName = builder.like(rootSubquery.<String> get("anonymous").get("name"), "%" + getAuthorName() + "%");
		Predicate predicateOnSurname = builder.like(rootSubquery.<String> get("anonymous").get("surname"), "%" + getAuthorSurname() + "%");
		Predicate predicateOnNameCompany = builder.like(rootSubquery.<String> get("companyName"), "%" + getAuthorCompanyName() + "%");

		subquery.where(builder.or(predicateOnName, predicateOnSurname, predicateOnNameCompany));

		return root.join("author").<Long> get("id").in(subquery);
	}

	private Predicate preparePredicateOnLoggedAuthor(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<Author> rootSubquery = subquery.from(Author.class);

		subquery.select(rootSubquery.<Long> get("id"));

		Predicate predicateOnName = builder.like(rootSubquery.join("loggedUser").<String> get("person").get("name"), "%" + getAuthorName() + "%");
		Predicate predicateOnSurname = builder.like(rootSubquery.join("loggedUser").<String> get("person").get("surname"), "%" + getAuthorSurname() + "%");
		Predicate predicateOnOutsourcer = builder.like(rootSubquery.join("loggedUser").join("outsourcer").<String> get("companyName"), "%" + getAuthorCompanyName() + "%");

		subquery.where(builder.or(predicateOnName, predicateOnSurname, predicateOnOutsourcer));

		return root.join("author").<Long> get("id").in(subquery);
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public RequestStatus getRequestStatus() {
		return requestStatus;
	}

	public void setRequestStatus(RequestStatus requestStatus) {
		this.requestStatus = requestStatus;
	}

	public SlaSchemaData getSlaSchema() {
		return slaSchema;
	}

	public void setSlaSchema(SlaSchemaData slaSchema) {
		this.slaSchema = slaSchema;
	}

	public ProductData getProduct() {
		return product;
	}

	public void setProduct(ProductData product) {
		this.product = product;
	}

	public String getAuthorName() {
		return authorName;
	}

	public void setAuthorName(String authorName) {
		this.authorName = authorName;
	}

	public String getAuthorSurname() {
		return authorSurname;
	}

	public void setAuthorSurname(String authorSurname) {
		this.authorSurname = authorSurname;
	}

	public String getAuthorCompanyName() {
		return authorCompanyName;
	}

	public void setAuthorCompanyName(String authorCompanyName) {
		this.authorCompanyName = authorCompanyName;
	}

	public String getSolver() {
		return solver;
	}

	public void setSolver(String solver) {
		this.solver = solver;
	}

	public Date getByCreatedDateTime() {
		return byCreatedDateTime;
	}

	public void setByCreatedDateTime(Date byCreatedDateTime) {
		this.byCreatedDateTime = byCreatedDateTime;
	}

	public Date getToCreatedDateTime() {
		return toCreatedDateTime;
	}

	public void setToCreatedDateTime(Date toCreatedDateTime) {
		this.toCreatedDateTime = toCreatedDateTime;
	}

	public Date getByReactionTime() {
		return byReactionTime;
	}

	public void setByReactionTime(Date byReactionTime) {
		this.byReactionTime = byReactionTime;
	}

	public Date getToReactionTime() {
		return toReactionTime;
	}

	public void setToReactionTime(Date toReactionTime) {
		this.toReactionTime = toReactionTime;
	}

	public Date getByTimeToSolve() {
		return byTimeToSolve;
	}

	public void setByTimeToSolve(Date byTimeToSolve) {
		this.byTimeToSolve = byTimeToSolve;
	}

	public Date getToTimeToSolve() {
		return toTimeToSolve;
	}

	public void setToTimeToSolve(Date toTimeToSolve) {
		this.toTimeToSolve = toTimeToSolve;
	}

	public Date getByDateOfClose() {
		return byDateOfClose;
	}

	public void setByDateOfClose(Date byDateOfClose) {
		this.byDateOfClose = byDateOfClose;
	}

	public Date getToDateOfClose() {
		return toDateOfClose;
	}

	public void setToDateOfClose(Date toDateOfClose) {
		this.toDateOfClose = toDateOfClose;
	}

}