package cz.tendersystems.helpdesk.service.general;

import java.io.*;
import java.nio.file.*;
import java.text.*;
import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.entity.request.Request;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.request.*;
import cz.tendersystems.helpdesk.service.security.*;

@RunWith(Arquillian.class)
public class FileServiceTest extends AbstractEJBTest {

	@EJB
	FileService fileService;

	@EJB
	ConfigurationService configurationService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();

		jar.addClasses(RequestService.class, RequestSecurityService.class, UserService.class, SlaSchemaService.class, OutsourcerService.class, ProductService.class, VersionType.class);
		jar.addClasses(PasswordService.class, TimeActivityService.class, MailService.class, EmbedTemplate.class, MailTemplateService.class, ConfigurationService.class, IdService.class);
		jar.addClasses(CommentService.class, FileService.class, ValidationRequest.class, ToolsForRequestDates.class, ToolsForHolidays.class);

		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@UsingDataSet("datasets/fileServiceTest/shouldFailToLoadFilesRequestOnSecurityException.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldFailToLoadFilesRequestOnSecurityException() {
		try {
			fileService.loadFilesRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist prilohy", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/fileServiceTest/shouldLoadFilesRequest.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldLoadFilesRequest() {
		login(LOGIN_NAME_ADMINISTRATOR);

		List<FileData> files = fileService.loadFilesRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertEquals(2, files.size());

	}

	@Test
	@UsingDataSet("datasets/fileServiceTest/shouldFailToSaveFileOnSecurityException.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldFailToSaveFileOnSecurityException() {
		Comment comment = em.find(Comment.class, 1l);
		comment.setRequest(em.find(Request.class, "1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));

		try {
			fileService.saveFile(new FileData(), comment);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni vytvorit prilohu", e.getMessage());
		}
	}

	@Test(expected = HelpdeskException.class)
	@UsingDataSet("datasets/fileServiceTest/shouldFailToSaveFileOnHelpdekExceptionWhileFileIsNull.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldFailToSaveFileOnHelpdekExceptionWhileFileIsNull() {
		Comment comment = em.find(Comment.class, 1l);
		comment.setRequest(em.find(Request.class, "1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));

		try {
			fileService.saveFile(null, comment);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Binarni data nesmi byt prazdna", e.getMessage());
			throw e;
		}
	}

	@Test(expected = HelpdeskException.class)
	@UsingDataSet("datasets/fileServiceTest/shouldFailToSaveFileOnHelpdekExceptionWhileInputStreamIsNull.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldFailToSaveFileOnHelpdekExceptionWhileInputStreamIsNull() {
		Comment comment = em.find(Comment.class, 1l);
		comment.setRequest(em.find(Request.class, "1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));

		try {
			fileService.saveFile(new FileData(), comment);
		} catch (HelpdeskException e) {
			Assert.assertEquals("File input stream nesmi byt prazdny", e.getMessage());
			throw e;
		}
	}

	@Test
	@ShouldMatchDataSet("datasets/fileServiceTest/file.xml")
	@UsingDataSet("datasets/fileServiceTest/shouldSaveFile.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldSaveFile() throws IOException {
		FileData data = new FileData();
		data.setContentType("text/plain");
		data.setName("Hello world");

		byte[] sampleFile = prepareSampleFile();
		Integer size = sampleFile.length;
		data.setSize(size.longValue());

		data.setInputStream(new ByteArrayInputStream(sampleFile));

		Comment comment = em.find(Comment.class, 1l);
		comment.setRequest(em.find(Request.class, "1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));

		fileService.saveFile(data, comment);

		Path file = Paths.get(configurationService.getBinaryFilesFolder() + prepareRelativePath() + "1");
		Assert.assertNotNull(file);

		byte[] result = Files.readAllBytes(file);
		Assert.assertTrue(new String(result, "UTF-8").equals("Hello world"));

		removeFileFromStorage();
	}

	@Test
	@ShouldMatchDataSet("datasets/fileServiceTest/file.xml")
	@UsingDataSet("datasets/fileServiceTest/shouldSaveFiles.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldSaveFiles() throws IOException {
		FileData data = new FileData();
		data.setContentType("text/plain");
		data.setName("Hello world");

		byte[] sampleFile = prepareSampleFile();
		Integer size = sampleFile.length;
		data.setSize(size.longValue());

		data.setInputStream(new ByteArrayInputStream(sampleFile));

		Comment comment = em.find(Comment.class, 1l);
		comment.setRequest(em.find(Request.class, "1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));

		List<FileData> files = new ArrayList<>();
		files.add(data);

		fileService.saveFiles(files, comment);

		Path file = Paths.get(configurationService.getBinaryFilesFolder() + prepareRelativePath() + "1");
		Assert.assertNotNull(file);

		byte[] result = Files.readAllBytes(file);
		Assert.assertTrue(new String(result, "UTF-8").equals("Hello world"));

		removeFileFromStorage();
	}

	@Test(expected = HelpdeskException.class)
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldFailToDownloadFileOnHelpdeskException() {
		try {
			fileService.downloadFile(1l);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Soubor 1 neexistuje", e.getMessage());

			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/fileServiceTest/shouldDownloadFile.xml")
	@ApplyScriptBefore("datasets/fileServiceTest/cleanup.sql")
	public void shouldDownloadFile() throws Exception {
		createFileForTest();

		FileData file = fileService.downloadFile(1l);

		Assert.assertNotNull(file);

		file.getInputStream().close();

		removeFileFromStorage();
	}

	private void removeFileFromStorage() throws IOException {
		Path path = Paths.get(configurationService.getBinaryFilesFolder() + prepareRelativePath() + "1");
		Files.delete(path);
	}

	private byte[] prepareSampleFile() {
		return "Hello world".getBytes();
	}

	private String prepareRelativePath() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		return MessageFormat.format("/{0}/{1}/{2}/", Integer.toString(cal.get(Calendar.YEAR)), Integer.toString(cal.get(Calendar.MONTH) + 1), Integer.toString(cal.get(Calendar.DAY_OF_MONTH)));
	}

	private void createFileForTest() throws Exception {
		transaction.begin();

		cz.tendersystems.helpdesk.entity.request.File data = new cz.tendersystems.helpdesk.entity.request.File();
		data.setContentType("text/plain");
		data.setName("Hello world");
		data.setCreatedDateTime(new Date());
		data.setSize(10l);
		data.setComment(em.find(Comment.class, 1l));
		em.persist(data);

		transaction.commit();

		String path = configurationService.getBinaryFilesFolder() + prepareRelativePath() + "1";
		Path pathToDir = Paths.get(path);
		Files.createDirectories(pathToDir.getParent());
		try {
			FileOutputStream fos = new FileOutputStream(path);
			fos.write(prepareSampleFile());
			fos.close();
		} catch (FileAlreadyExistsException e) {
			System.out.println("File already exist");
		}

	}

}
