package cz.tendersystems.helpdesk.service.log.list;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.general.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.list.Filter;

@RunWith(Arquillian.class)
public class ApplicationLogListServiceTest extends AbstractEJBTest {

	@EJB
	ApplicationLogListService applicationLogListService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(ApplicationLogListService.class, AbstractListService.class, RequestList.class, Filter.class, Response.class, ApplicationLogFilter.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFailToFindOnSecurityException() {
		try {
			applicationLogListService.find(null);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam logu", e.getMessage());
		}
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindAllApplicationLogs() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnId() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setId(4l);

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnDate() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setByDate(prepareDate("2010-01-01 00.00.00"));
		filter.setToDate(prepareDate());

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnUser() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setUser("a");

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnOperation() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setOperation("varování");

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnDetails() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setOperation("logu");

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(5, response.getData().size());
		Assert.assertEquals(Long.valueOf("6"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnIdRequest() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setIdRequest("1");

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnLevel() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setLevel(LevelRecord.ERROR);

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/applicationLogListServiceTest/applicationLogs.xml", "datasets/applicationLogListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/applicationLogListServiceTest/cleanup.sql")
	public void shouldFindWithCombinationFilter() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ApplicationLogFilter filter = new ApplicationLogFilter();
		filter.setLevel(LevelRecord.INFORMATION);
		filter.setByDate(prepareDate("2000-01-01 00.00.00"));
		filter.setToDate(prepareDate());

		Response<ApplicationLogData> response = applicationLogListService.find(new RequestList<ApplicationLogFilter>(filter, 1, 10));

		Assert.assertEquals(4, response.getData().size());
		Assert.assertEquals(Long.valueOf("5"), response.getRecordCounter());
	}
}
