package cz.tendersystems.helpdesk.service.organizationalstructure;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;

@RunWith(Arquillian.class)
public class OutsourcerServiceTest extends AbstractEJBTest {

	@EJB
	OutsourcerService outsourcerService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClass(OutsourcerService.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailFindOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		try {
			outsourcerService.findOutsourcer(ID_NOT_EXIST_OBJECT);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Zadavatel 0 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToFindOutsourcerOnSecurityException() {
		try {
			outsourcerService.findOutsourcer(ID_NOT_EXIST_OBJECT);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni pripravit zadavatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/outsourcerServiceTest/outsourcer.xml")
	@ApplyScriptBefore("datasets/outsourcerServiceTest/cleanup.sql")
	public void shouldFindOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Long id = 1l;
		Outsourcer outsourcer = outsourcerService.findOutsourcer(1l);

		Assert.assertNotNull(outsourcer);
		Assert.assertEquals(id, outsourcer.getId());
		Assert.assertNotNull(outsourcer.getCompanyName());
		Assert.assertNotNull(outsourcer.getCreatedDateTime());
		Assert.assertTrue(outsourcer.getActive());
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailToLoadOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		try {
			outsourcerService.loadOutsourcer(ID_NOT_EXIST_OBJECT);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Zadavatel 0 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToLoadOutsourcerOnSecurityException() {
		try {
			outsourcerService.loadOutsourcer(ID_NOT_EXIST_OBJECT);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni pripravit zadavatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/outsourcerServiceTest/outsourcer.xml")
	@ApplyScriptBefore("datasets/outsourcerServiceTest/cleanup.sql")
	public void shouldLoadOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Long id = 1l;
		OutsourcerData outsourcer = outsourcerService.loadOutsourcer(1l);

		Assert.assertNotNull(outsourcer);
		Assert.assertEquals(id, outsourcer.getId());
		Assert.assertNotNull(outsourcer.getCompanyName());
		Assert.assertNotNull(outsourcer.getCreatedDateTime());
		Assert.assertTrue(outsourcer.getActive());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToCreateOutsourcerOnSecurityException() {
		try {
			outsourcerService.createOutsourcer(new OutsourcerData());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni vytvorit zadavatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@ShouldMatchDataSet(value = "datasets/outsourcerServiceTest/outsourcer-edit.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldCreateOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerData outsourcer = new OutsourcerData();
		outsourcer.setCompanyName("Tendersystems");
		outsourcer.setCreatedDateTime(prepareDate());
		outsourcer.setActive(true);

		outsourcerService.createOutsourcer(outsourcer);
		checkApplicationLog("Vytvoření zadavatele", "Vytvoření zadavatele s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToUpdateOutsourcerOnSecurityException() {
		try {
			outsourcerService.updateOutsourcer(new OutsourcerData());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit zadavatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/outsourcerServiceTest/outsourcer.xml")
	@ShouldMatchDataSet(value = "datasets/outsourcerServiceTest/outsourcer-edit.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldUpdateOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerData outsourcer = new OutsourcerData(em.find(Outsourcer.class, 1L));

		outsourcer.setCompanyName("Tendersystems");
		outsourcerService.updateOutsourcer(outsourcer);

		checkApplicationLog("Upravení zadavatele", "Upravení zadavatele s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToUpdateActivityOfOutsourcerAndsUsersOnSecurityException() {
		try {
			outsourcerService.updateActivityOfOutsourcerAndUsers(ID_NOT_EXIST_OBJECT, false);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit aktivitu zadavatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/outsourcerServiceTest/activityOutsourcer.xml")
	@ShouldMatchDataSet(value = "datasets/outsourcerServiceTest/activityOutsourcer-edit.xml")
	@ApplyScriptBefore("datasets/outsourcerServiceTest/cleanup.sql")
	public void shouldUpdateActivityOfOutsourcerAndUsers() {
		login(LOGIN_NAME_ADMINISTRATOR);

		outsourcerService.updateActivityOfOutsourcerAndUsers(1l, false);

		checkApplicationLog("Změna aktivity zadavatele", "Změna aktivity zadavatele s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@UsingDataSet("datasets/outsourcerServiceTest/activityOutsourcer.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadIdOutsourcerUser() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertEquals(1l, outsourcerService.loadIdOutsourcerUser("uzivatelz1"));
	}
}
