package cz.tendersystems.helpdesk.service.organizationalstructure;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.mail.*;

@RunWith(Arquillian.class)
public class PasswordServiceTest extends AbstractEJBTest {

	private static final String loginName = "uzivatelTa";
	private static final String password = "pass1234";

	@EJB
	PasswordService passwordService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(PasswordService.class, ValidationPassword.class, MailTemplateService.class, EmbedTemplate.class, ConfigurationService.class, VersionType.class, MailService.class, DateAndTimeFormat.class);
		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	public void shouldFailToChangePassword() {
		try {
			passwordService.changePassword("", password);
		} catch (ValidationException e) {
			Assert.assertEquals("error.keyIsInvalid", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/passwordServiceTest/password.xml")
	@ShouldMatchDataSet(value = "datasets/passwordServiceTest/password-edit.xml")
	@ApplyScriptBefore("datasets/passwordServiceTest/cleanup.sql")
	public void shouldChangePassword() throws ValidationException {
		String password = "pass1234A@";

		passwordService.changePassword("8165469194", password);

		checkApplicationLog("Změna hesla", "Změna hesla uživatele", loginName, null);
	}

	@Test
	public void shouldFailToSendLinkForChangePasswordWhenUserIsNull() {
		try {
			passwordService.sendLinkForChangePassword(SOLVER_LOGIN_NAME);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userNotExistOrIsFormidden", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/passwordServiceTest/forbiddenUser.xml")
	@ApplyScriptBefore("datasets/passwordServiceTest/cleanup.sql")
	public void shouldFailToSendLinkForChangePasswordWhenUserIsForbidden() {
		try {
			passwordService.sendLinkForChangePassword(loginName);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userNotExistOrIsFormidden", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/passwordServiceTest/password-edit.xml")
	@ApplyScriptBefore("datasets/passwordServiceTest/cleanup.sql")
	public void shouldSendLinkForChangePassword() throws ValidationException {
		passwordService.sendLinkForChangePassword(loginName);

		SystemUser user = em.find(SystemUser.class, loginName);

		Assert.assertNotNull(user.getChangePassword().getKey());
		Assert.assertNotNull(user.getChangePassword().getExpirationDate());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToSendLinkForFirstConfigurationPassword() {
		try {
			passwordService.sendLinkForFirstConfigurationPassword(new UserSolver());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni odeslat odkaz pro prvotni nastaveni hesla", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/passwordServiceTest/password.xml")
	@ApplyScriptBefore("datasets/passwordServiceTest/cleanup.sql")
	public void shouldSendLinkForFirstConfigurationPassword() {
		login(LOGIN_NAME_ADMINISTRATOR);

		passwordService.sendLinkForFirstConfigurationPassword(em.find(UserOutsourcer.class, loginName));

		UserOutsourcer user = em.find(UserOutsourcer.class, loginName);

		Assert.assertNotNull(user.getChangePassword().getKey());
		Assert.assertNotNull(user.getChangePassword().getExpirationDate());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailTOPreparePasswordHashOnSecurityException() {
		try {
			passwordService.preparePasswordHash(loginName);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni pripravit hash hesla", e.getMessage());
			throw e;
		}
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailToPreparePasswordHash() {
		login(LOGIN_NAME_ADMINISTRATOR);

		try {
			passwordService.preparePasswordHash(null);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Prihlasovaci jmeno nemuze byt null", e.getMessage());
			throw e;
		}
	}

	@Test
	public void shouldPreparePasswordHash() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertNotNull(passwordService.preparePasswordHash(loginName));
	}

	@Test
	public void shouldCheckTheValidityOfKeyReturnFalse() {
		Assert.assertFalse(passwordService.checkTheValidityOfKey(loginName));
	}

	@Test
	@UsingDataSet("datasets/passwordServiceTest/password.xml")
	@ApplyScriptBefore("datasets/passwordServiceTest/cleanup.sql")
	public void shouldCheckTheValidityOfKeyReturnTrue() {
		Assert.assertTrue(passwordService.checkTheValidityOfKey("8165469194"));
	}
}
