package cz.tendersystems.helpdesk.service.organizationalstructure;

import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;

@RunWith(Arquillian.class)
public class UserServiceTest extends AbstractEJBTest {

	private static final String outsourcerLoginName = "uzivatelTa";
	private static final String solverLoginName = "resitel";
	private static final String notExistLoginName = "uzivatelz1";

	@EJB
	UserService userService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(PasswordService.class, ValidationPassword.class, MailTemplateService.class, EmbedTemplate.class, ConfigurationService.class, VersionType.class, MailService.class, DateAndTimeFormat.class);
		jar.addClasses(UserService.class, OutsourcerService.class, ProductService.class, ValidationUser.class);
		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailToFindUser() throws Exception {
		try {
			userService.findUser(notExistLoginName);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Uzivatel s prihlasovacim jmenem uzivatelz1 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userOutsourcer.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldFindUserOutsourcer() {
		UserOutsourcer user = (UserOutsourcer) userService.findUser(outsourcerLoginName);

		Assert.assertEquals(outsourcerLoginName, user.getLoginName());
		Assert.assertNotNull(user.getPerson());
		Assert.assertEquals(RoleType.OUTSOURCER, user.getRoleType());
		Assert.assertNotNull(user.getChangePassword());
		Assert.assertNotNull(user.getOutsourcer());
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userSolver.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldFindUserSolver() {
		UserSolver user = (UserSolver) userService.findUser(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertEquals(LOGIN_NAME_ADMINISTRATOR, user.getLoginName());
		Assert.assertNotNull(user.getPerson());
		Assert.assertEquals(RoleType.ADMINISTRATOR, user.getRoleType());
		Assert.assertNotNull(user.getChangePassword());
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailToLoadUser() throws Exception {
		try {
			userService.loadUser(notExistLoginName);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Uzivatel s prihlasovacim jmenem uzivatelz1 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userOutsourcer.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadUserOutsourcer() {
		UserOutsourcerData user = (UserOutsourcerData) userService.loadUser(outsourcerLoginName);

		Assert.assertEquals(outsourcerLoginName, user.getLoginName());
		Assert.assertNotNull(user.getPerson());
		Assert.assertEquals(RoleType.OUTSOURCER, user.getRoleType());
		Assert.assertNotNull(user.getChangePassword());
		Assert.assertNotNull(user.getOutsourcer());
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userSolver.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadUserSolver() {
		UserSolverData user = (UserSolverData) userService.loadUser(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertEquals(LOGIN_NAME_ADMINISTRATOR, user.getLoginName());
		Assert.assertNotNull(user.getPerson());
		Assert.assertEquals(RoleType.ADMINISTRATOR, user.getRoleType());
		Assert.assertNotNull(user.getChangePassword());
		Assert.assertTrue(user.getProducts().isEmpty());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToCreateUserOnSecurityException() throws ValidationException {
		try {
			userService.createUser(new UserSolverData());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni vytvorit uzivatele", e.getMessage());
			throw e;
		}
	}

	@Test
	public void shoulFailToCreateUserWhileUserIsNotProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserSolverData user = new UserSolverData();
		user.setRoleType(RoleType.SOLVER);

		try {
			userService.createUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userMustBeProduct", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userSolver.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shoulFailToCreateUserWhileLoginNameIsNotAvaible() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserSolverData user = new UserSolverData();
		user.setLoginName("administrator");

		try {
			userService.createUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.loginNameIsNotAvailable", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/outsourcer.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldCreateUserOutsourcer() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserOutsourcerData user = new UserOutsourcerData();
		prepareUser(user, outsourcerLoginName, RoleType.OUTSOURCER);
		user.setOutsourcer(new OutsourcerData(prepareOutsourcer()));

		userService.createUser(user);

		checkUser((UserOutsourcerData) userService.loadUser(outsourcerLoginName), outsourcerLoginName, RoleType.OUTSOURCER);
		checkApplicationLog("Vytvoření uživatele", "Vytvoření uživatele s přihlašovacím jménem uzivatelTa", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldCreateAdministrator() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserSolverData user = new UserSolverData();
		prepareUser(user, LOGIN_NAME_ADMINISTRATOR, RoleType.ADMINISTRATOR);

		userService.createUser(user);

		checkUser((UserSolverData) userService.loadUser(LOGIN_NAME_ADMINISTRATOR), LOGIN_NAME_ADMINISTRATOR, RoleType.ADMINISTRATOR);
		checkApplicationLog("Vytvoření uživatele", "Vytvoření uživatele s přihlašovacím jménem administrator", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/product.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldCreateSolver() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserSolverData user = new UserSolverData();
		prepareUser(user, solverLoginName, RoleType.SOLVER);
		user.setProducts(Arrays.asList(new ProductData(prepareProduct())));

		userService.createUser(user);

		checkUser((UserSolverData) userService.loadUser(solverLoginName), solverLoginName, RoleType.SOLVER);
		checkApplicationLog("Vytvoření uživatele", "Vytvoření uživatele s přihlašovacím jménem resitel", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToUpdateUserOnSecurityException() throws ValidationException {
		try {
			userService.updateUser(new UserSolverData());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit uzivatele", e.getMessage());
			throw e;
		}
	}

	@Test
	public void shouldFailToUpdateUserOnValidationException() {
		login(LOGIN_NAME_ADMINISTRATOR);

		try {
			UserSolverData user = new UserSolverData();
			user.setRoleType(RoleType.OUTSOURCER);
			userService.updateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userSolverCanNotBeRoleTypeOutsourcer", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userOutsourcer.xml")
	@ShouldMatchDataSet("datasets/userServiceTest/userOutsourcer-update.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldUpdateUserOutsourcer() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserOutsourcerData user = (UserOutsourcerData) userService.loadUser(outsourcerLoginName);
		user.getPerson().setTelephone("+420604931807");

		userService.updateUser(user);

		checkApplicationLog("Upravení uživatele", "Upravení uživatele s přihlašovacím jménem uzivatelTa", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@UsingDataSet({ "datasets/userServiceTest/userSolver.xml", "datasets/userServiceTest/product.xml" })
	@ShouldMatchDataSet("datasets/userServiceTest/userSolver-update.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldUpdateUserSolver() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserSolverData user = (UserSolverData) userService.loadUser(LOGIN_NAME_ADMINISTRATOR);
		user.setRoleType(RoleType.SOLVER);
		user.setProducts(Arrays.asList(new ProductData(prepareProduct())));

		userService.updateUser(user);

		checkApplicationLog("Upravení uživatele", "Upravení uživatele s přihlašovacím jménem administrator", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToChangeUserAcountStatusOnSecurityException() throws ValidationException {
		try {
			userService.changeUserAcountStatus(LOGIN_NAME_ADMINISTRATOR, UserAccountStatus.ENABLED);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni zmenit stav uzivatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userSolver.xml")
	@ShouldMatchDataSet("datasets/userServiceTest/activityOfSolver.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldChangeUserAcountStatus() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);
		userService.changeUserAcountStatus(LOGIN_NAME_ADMINISTRATOR, UserAccountStatus.FORBIDDEN);

		checkApplicationLog("Změna stavu uživatele", "Změna stavu uživatele s přihlašovacím jménem administrator", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToLoadUserOfProductOnSecurityException() throws ValidationException {
		try {
			userService.loadUserOfProduct(1l);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni pripravit uzivatele produktu", e.getMessage());
			throw e;
		}
	}

	@Test
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadUserOfProductEmptyList() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(userService.loadUserOfProduct(1l).isEmpty());
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userSolver-update.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadUserOfProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(!userService.loadUserOfProduct(1l).isEmpty());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToLoadUserOfOutsourcerOnSecurityException() throws ValidationException {
		try {
			userService.loadUserOfOutsourcer(1l);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni pripravit uzivatele zadavatele", e.getMessage());
			throw e;
		}
	}

	@Test
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadUserOfOutsourcerEmptyList() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(userService.loadUserOfOutsourcer(1l).isEmpty());
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userOutsourcer-update.xml")
	@ApplyScriptBefore("datasets/userServiceTest/cleanup.sql")
	public void shouldLoadUserOfOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(!userService.loadUserOfOutsourcer(1l).isEmpty());
	}

	private void checkUser(SystemUserData user, String loginName, RoleType roleType) {
		Assert.assertEquals(loginName, user.getLoginName());
		Assert.assertNotNull(user.getChangePassword());
		Assert.assertEquals(preparePerson().toString(), new Person(user.getPerson()).toString());
		Assert.assertEquals(roleType, user.getRoleType());
	}

	private void prepareUser(SystemUserData user, String loginName, RoleType roleType) {
		user.setLoginName(loginName);
		user.setCreatedDateTime(prepareDate());
		user.setRoleType(roleType);
		user.setUserAccountStatus(UserAccountStatus.ENABLED);
		user.setPerson(new PersonData(preparePerson()));
	}

	private Person preparePerson() {
		Person person = new Person();
		person.setName("Zbyněk");
		person.setSurname("Richter");
		person.setMail("zbytos@gmail.com");

		return person;
	}

	private Outsourcer prepareOutsourcer() {
		Outsourcer outsourcer = new Outsourcer();

		outsourcer.setId(1l);
		outsourcer.setCompanyName("Ample");
		outsourcer.setCreatedDateTime(prepareDate());

		outsourcer.setActive(true);

		return outsourcer;
	}

	private Product prepareProduct() {
		Product product = new Product();

		product.setId(1l);
		product.setName("eozpf");
		product.setActive(true);

		return product;
	}
}
