package cz.tendersystems.helpdesk.service.organizationalstructure.list;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.list.Filter;

@RunWith(Arquillian.class)
public class UserListServiceTest extends AbstractEJBTest {

	@EJB
	UserListService userListService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(UserListService.class, AbstractListService.class, RequestList.class, Filter.class, Response.class, UserFilter.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFailToFindOnSecurityException() {
		try {
			userListService.find(null);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam uzivatelu", e.getMessage());
		}
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindAllProducts() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnLoginName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setLoginName("Ta");

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setName("Zby");

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnUsername() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setSurname("ri");

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("0"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnMail() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setMail("zby");

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnTelephone() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setTelephone(222l);

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnRoleType() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setRoleType(RoleType.ADMINISTRATOR);

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnUserAccountStatus() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setUserAccountStatus(UserAccountStatus.FORBIDDEN);

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnDate() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setByDate(prepareDate("2012-02-05 00.00.00"));
		filter.setToDate(prepareDate());

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(5, response.getData().size());
		Assert.assertEquals(Long.valueOf("6"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/userListServiceTest/users.xml", "datasets/userListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/userListServiceTest/cleanup.sql")
	public void shouldFindWithCombinationFilter() {
		login(LOGIN_NAME_ADMINISTRATOR);

		UserFilter filter = new UserFilter();
		filter.setByDate(prepareDate("2012-02-05 00.00.00"));
		filter.setToDate(prepareDate());
		filter.setUserAccountStatus(UserAccountStatus.ENABLED);
		filter.setRoleType(RoleType.OUTSOURCER);

		Response<SystemUserData> response = userListService.find(new RequestList<UserFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}
}
