package cz.tendersystems.helpdesk.service.organizationalstructure.sla;

import java.text.*;
import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;

@RunWith(Arquillian.class)
public class ProductServiceTest extends AbstractEJBTest {

	@EJB
	ProductService productService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClass(ProductService.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailFindProduct() {
		try {
			productService.findProduct(ID_NOT_EXIST_OBJECT);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Produkt 0 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/productServiceTest/product.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldFindProduct() throws Exception {
		Long id = 1l;
		Product product = productService.findProduct(id);

		Assert.assertNotNull(product);
		Assert.assertEquals(id, product.getId());
		Assert.assertNotNull(product.getName());
		Assert.assertEquals(true, product.getActive());
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailLoadProduct() {
		try {
			productService.loadProduct(ID_NOT_EXIST_OBJECT);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Produkt 0 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/productServiceTest/product.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldLoadProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Long id = 1l;
		ProductData product = productService.loadProduct(1l);

		Assert.assertNotNull(product);
		Assert.assertEquals(id, product.getId());
		Assert.assertNotNull(product.getName());
		Assert.assertEquals(true, product.getActive());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToCreateProductOnSecurityException() {
		login(SOLVER_LOGIN_NAME);

		try {
			productService.createProduct(new ProductData());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni vytvorit produkt", e.getMessage());
			throw e;
		}
	}

	@Test
	@ShouldMatchDataSet(value = "datasets/productServiceTest/product-edit.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldCreateProduct() throws ParseException {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductData product = new ProductData();
		product.setName("ta");
		product.setActive(true);

		productService.createProduct(product);

		checkApplicationLog("Vytvoření produktu", "Vytvoření produktu s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToUpdateProductOnSecurityException() {
		login(SOLVER_LOGIN_NAME);

		try {
			productService.updateProduct(new ProductData());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit produkt", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/productServiceTest/product.xml")
	@ShouldMatchDataSet(value = "datasets/productServiceTest/product-edit.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldUpdateProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductData product = new ProductData(em.find(Product.class, 1L));

		product.setName("ta");
		productService.updateProduct(product);
		checkApplicationLog("Upravení produktu", "Upravení produktu s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToUpdateActivityOfProductOnSecurityException() {
		login(SOLVER_LOGIN_NAME);

		try {
			productService.updateActivityOfProduct(1L, false);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit aktivitu produktu", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/productServiceTest/activityOfProduct.xml")
	@ShouldMatchDataSet(value = "datasets/productServiceTest/activityOfProduct-edit.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldUpdateActivityOfProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		productService.updateActivityOfProduct(1l, false);

		checkApplicationLog("Změna aktivity produktu", "Změna aktivity produktu s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldLoadIdProductUserEmptyList() {
		Assert.assertTrue(productService.loadIdProductUser(LOGIN_NAME_ADMINISTRATOR).isEmpty());
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/userSolver.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldLoadIdProductUser() {
		Assert.assertTrue(!productService.loadIdProductUser(LOGIN_NAME_ADMINISTRATOR).isEmpty());
	}

	@Test
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldLoadProductsEmptyList() {
		Assert.assertTrue(productService.loadIdProductUser(LOGIN_NAME_ADMINISTRATOR).isEmpty());
	}

	@Test
	@UsingDataSet("datasets/userServiceTest/product.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldLoadProducts() {
		Long id = 1l;

		List<ProductData> products = productService.loadProducts();

		Assert.assertEquals(1, products.size());

		for (ProductData product : products) {
			Assert.assertEquals(id, product.getId());
			Assert.assertEquals("eozpf", product.getName());
			Assert.assertTrue(product.getActive());
		}
	}
}
