package cz.tendersystems.helpdesk.service.organizationalstructure.sla;

import java.text.*;
import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;

@RunWith(Arquillian.class)
public class SlaSchemaServiceTest extends AbstractEJBTest {

	@EJB
	SlaSchemaService slaSchemaService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();

		jar.addClasses(UserService.class, SlaSchemaService.class, TimeActivityService.class, ValidationSlaSchema.class, ProductService.class, OutsourcerService.class);
		jar.addClasses(PasswordService.class, MailService.class, MailTemplateService.class, EmbedTemplate.class, ConfigurationService.class, VersionType.class);
		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@InSequence(1)
	@UsingDataSet("datasets/slaSchemaServiceTest/slaSchema.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldFindSlaSchema() {
		Long id = 1l;
		SlaSchema slaSchema = slaSchemaService.findSlaSchema(id);

		Assert.assertNotNull(slaSchema);
		Assert.assertEquals(id, slaSchema.getId());
		Assert.assertNotNull(slaSchema.getName());
		Assert.assertNotNull(slaSchema.getReactionTime());
		Assert.assertNotNull(slaSchema.getTimeToSolve());
		Assert.assertTrue(slaSchema.getActive());
	}

	@Test(expected = HelpdeskException.class)
	@InSequence(2)
	public void shouldFindToLoadSlaSchema() {
		try {
			slaSchemaService.findSlaSchema(ID_NOT_EXIST_OBJECT);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Sla schema 0 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test
	@InSequence(4)
	@UsingDataSet("datasets/slaSchemaServiceTest/slaSchema.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldLoadSlaSchema() {
		Long id = 1l;
		SlaSchemaData slaSchema = slaSchemaService.loadSlaSchema(id);

		Assert.assertNotNull(slaSchema);
		Assert.assertEquals(id, slaSchema.getId());
		Assert.assertNotNull(slaSchema.getName());
		Assert.assertNotNull(slaSchema.getReactionTime());
		Assert.assertNotNull(slaSchema.getTimeToSolve());
		Assert.assertTrue(slaSchema.getActive());
	}

	@Test(expected = HelpdeskException.class)
	@InSequence(4)
	public void shouldFailToLoadSlaSchema() {
		try {
			slaSchemaService.loadSlaSchema(ID_NOT_EXIST_OBJECT);
		} catch (HelpdeskException e) {
			Assert.assertEquals("Sla schema 0 neexistuje", e.getMessage());
			throw e;
		}
	}

	@Test
	@InSequence(5)
	@UsingDataSet({ "datasets/slaSchemaServiceTest/productSlaSchema.xml", "datasets/slaSchemaServiceTest/slaSchema.xml", "datasets/slaSchemaServiceTest/assignmentWithoutOutsourcer.xml" })
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldLoadAssignmentSlaSchema() {
		Long id = 1l;

		AssignmentData assignment = slaSchemaService.loadAssignmentSlaSchema(id);

		Assert.assertNotNull(assignment.getSlaSchema());
		Assert.assertEquals(id, assignment.getSlaSchema().getId());
		Assert.assertNotNull(assignment.getSlaSchema().getName());
		Assert.assertNotNull(assignment.getSlaSchema().getReactionTime());
		Assert.assertNotNull(assignment.getSlaSchema().getTimeToSolve());
		Assert.assertTrue(assignment.getSlaSchema().getActive());
	}

	@Test
	@InSequence(6)
	public void shouldFailToLoadSchemasOnSecurityExceptio() {
		try {
			slaSchemaService.loadSlaSchemas();
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam sla schema", e.getMessage());
		}
	}

	@Test
	@InSequence(7)
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldLoadSlaSchemasEmptyList() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(slaSchemaService.loadSlaSchemas().isEmpty());
	}

	@Test
	@InSequence(8)
	@UsingDataSet("datasets/slaSchemaServiceTest/slaSchema.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldLoadSchemas() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(!slaSchemaService.loadSlaSchemas().isEmpty());
	}

	@Test
	@InSequence(9)
	@UsingDataSet("datasets/slaSchemaServiceTest/productSlaSchema.xml")
	@ShouldMatchDataSet("datasets/slaSchemaServiceTest/assignmentWithoutOutsourcer-create.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldCreateSlaSchemaWithoutOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		AssignmentData assignment = new AssignmentData();
		TimeActivityData timeActivity = new TimeActivityData();

		prepareAssignment(assignment, timeActivity);

		try {
			slaSchemaService.createSlaSchema(assignment, Arrays.asList(timeActivity));

			checkApplicationLog("Vytvoření sla schéma", null, LOGIN_NAME_ADMINISTRATOR, null);
		} catch (ValidationException e) {
			e.printStackTrace();
		}
	}

	@Test
	@InSequence(10)
	@UsingDataSet({ "datasets/slaSchemaServiceTest/productSlaSchema.xml", "datasets/slaSchemaServiceTest/outsourcerSlaSchema.xml" })
	@ShouldMatchDataSet("datasets/slaSchemaServiceTest/assignmentWithOutsourcer-create.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldCreateAssignmentWithOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		AssignmentData assignment = new AssignmentData();
		TimeActivityData timeActivity = new TimeActivityData();

		prepareAssignment(assignment, timeActivity);

		assignment.setOutsourcer(new OutsourcerData(em.find(Outsourcer.class, 1L)));

		try {
			slaSchemaService.createSlaSchema(assignment, Arrays.asList(timeActivity));

			checkApplicationLog("Vytvoření sla schéma", null, LOGIN_NAME_ADMINISTRATOR, null);
		} catch (ValidationException e) {
			e.printStackTrace();
		}
	}

	@Test
	@InSequence(11)
	public void shouldFailToCreateSlaSchemaOnSecurityException() throws ValidationException {
		login(SOLVER_LOGIN_NAME);

		try {
			slaSchemaService.createSlaSchema(new AssignmentData(), new ArrayList<>());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni vytvorit sla schema", e.getMessage());
		}
	}

	@Test
	@InSequence(12)
	@UsingDataSet({ "datasets/slaSchemaServiceTest/productSlaSchema.xml", "datasets/slaSchemaServiceTest/slaSchema.xml", "datasets/slaSchemaServiceTest/assignmentWithoutOutsourcer.xml" })
	@ShouldMatchDataSet("datasets/slaSchemaServiceTest/assignmentWithoutOutsourcer-edit.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldUpdateSlaSchemaWithoutOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		AssignmentData assignment = new AssignmentData((Assignment) em.createQuery("SELECT a FROM Assignment a LEFT JOIN FETCH a.product LEFT JOIN FETCH a.slaSchema LEFT JOIN FETCH a.outsourcer WHERE a.id = :idAssignment").setParameter("idAssignment", 1L).getSingleResult());
		assignment.getSlaSchema().setName("sla2");

		TimeActivityData timeActivity = new TimeActivityData();
		timeActivity.setDayName(DayName.THURSDAY);
		prepareDatesOfTimesActivity(timeActivity);

		try {
			slaSchemaService.updateSlaSchema(assignment, Arrays.asList(timeActivity));

			checkApplicationLog("Upravení sla schéma", null, LOGIN_NAME_ADMINISTRATOR, null);
		} catch (ValidationException e) {
			e.printStackTrace();
		}
	}

	@Test
	@InSequence(13)
	@UsingDataSet({ "datasets/slaSchemaServiceTest/productSlaSchema.xml", "datasets/slaSchemaServiceTest/outsourcerSlaSchema.xml", "datasets/slaSchemaServiceTest/slaSchema.xml", "datasets/slaSchemaServiceTest/assignmentWithOutsourcer.xml" })
	@ShouldMatchDataSet("datasets/slaSchemaServiceTest/assignmentWithOutsourcer-edit.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldUpdateSlaSchemaWithOutsourcer() {
		login(LOGIN_NAME_ADMINISTRATOR);

		AssignmentData assignment = new AssignmentData((Assignment) em.createQuery("SELECT a FROM Assignment a LEFT JOIN FETCH a.product LEFT JOIN FETCH a.slaSchema LEFT JOIN FETCH a.outsourcer WHERE a.id = :idAssignment").setParameter("idAssignment", 1L).getSingleResult());
		assignment.getSlaSchema().setName("sla2");

		TimeActivityData timeActivity = new TimeActivityData();
		timeActivity.setDayName(DayName.THURSDAY);
		prepareDatesOfTimesActivity(timeActivity);

		try {
			slaSchemaService.updateSlaSchema(assignment, Arrays.asList(timeActivity));

			checkApplicationLog("Upravení sla schéma", null, LOGIN_NAME_ADMINISTRATOR, null);

		} catch (ValidationException e) {
			e.printStackTrace();
		}
	}

	@Test
	@InSequence(14)
	public void shouldFailToUpdateSlaSchemaOnSecurityService() throws ValidationException {
		login(SOLVER_LOGIN_NAME);

		try {
			slaSchemaService.updateSlaSchema(new AssignmentData(), new ArrayList<>());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit sla schema", e.getMessage());
		}
	}

	@Test
	@InSequence(15)
	@UsingDataSet("datasets/slaSchemaServiceTest/slaSchema.xml")
	@ShouldMatchDataSet("datasets/slaSchemaServiceTest/slaSchema-edit.xml")
	@ApplyScriptBefore("datasets/slaSchemaServiceTest/cleanup.sql")
	public void shouldUpdateActivityOfSlaSchema() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaData slaSchema = new SlaSchemaData();
		slaSchema.setId(1L);

		slaSchemaService.updateActivityOfSlaSchema(slaSchema, false);

		checkApplicationLog("Změna aktivity sla schéma", "Změna aktivity sla schéma s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@InSequence(16)
	public void shouldFailToUpdateActivityOfSlaSchemaOnSecurityException() throws ValidationException {
		login(SOLVER_LOGIN_NAME);

		try {
			slaSchemaService.updateActivityOfSlaSchema(new SlaSchemaData(), false);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit aktivitu sla schema", e.getMessage());
		}
	}

	private void prepareAssignment(AssignmentData assignment, TimeActivityData timeActivity) {

		SlaSchemaData slaSchema = new SlaSchemaData();
		slaSchema.setName("sla1");

		try {
			slaSchema.setReactionTime(new SimpleDateFormat("hh:mm:ss").parse("08:00:00"));
			slaSchema.setTimeToSolve(new SimpleDateFormat("hh:mm:ss").parse("08:00:00"));
		} catch (ParseException e) {
			e.printStackTrace();
		}

		timeActivity.setDayName(DayName.HOLIDAY);
		prepareDatesOfTimesActivity(timeActivity);

		assignment.setSlaSchema(slaSchema);
		assignment.setProduct(new ProductData(em.find(Product.class, 1L)));
	}

	private void prepareDatesOfTimesActivity(TimeActivityData timeActivity) {
		try {
			timeActivity.setStartTime(new SimpleDateFormat("hh:mm:ss").parse("08:00:00"));
			timeActivity.setEndTime(new SimpleDateFormat("hh:mm:ss").parse("16:00:00"));
		} catch (ParseException e) {
			e.printStackTrace();
		}
	}
}
