package cz.tendersystems.helpdesk.service.request;

import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.security.*;

@RunWith(Arquillian.class)
public class CommentServiceTest extends AbstractEJBTest {

	@EJB
	CommentService commentService;

	@EJB
	RequestService requestService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();

		jar.addClasses(RequestService.class, RequestSecurityService.class, UserService.class, SlaSchemaService.class, OutsourcerService.class, ProductService.class, VersionType.class);
		jar.addClasses(PasswordService.class, TimeActivityService.class, MailService.class, EmbedTemplate.class, MailTemplateService.class, ConfigurationService.class, IdService.class);
		jar.addClasses(CommentService.class, FileService.class, ValidationRequest.class, ToolsForRequestDates.class, ToolsForHolidays.class);

		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldFailToLoadCommentRequestOnSecurityException.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldFailToLoadCommentRequestOnSecurityException() {
		try {
			commentService.loadCommentRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist pozadavek", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldLoadEmptyCollection.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldLoadEmptyCollection() {
		login(LOGIN_NAME_ADMINISTRATOR);

		List<CommentData> comments = commentService.loadCommentRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertTrue(comments.isEmpty());
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldLoadComments.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldLoadComments() {
		login(LOGIN_NAME_ADMINISTRATOR);

		List<CommentData> comments = commentService.loadCommentRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertEquals(2, comments.size());
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldCreateCommentAtCreateRequest.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldCreateCommentAtCreateRequest() {
		cz.tendersystems.helpdesk.entity.request.Request request = requestService.findRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");
		request.setSlaSchema(em.find(SlaSchema.class, 1l));
		request.setAuthor(em.find(Author.class, 1l));

		commentService.createCommentAtCreateRequest(request, prepareComment(), new ArrayList<>());

		List<CommentData> comments = commentService.loadCommentRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		for (CommentData c : comments) {
			Assert.assertEquals("zbytos@gmail.com", c.getAuthor());
			Assert.assertNotNull(c.getCreatedDateTime());
			Assert.assertEquals(RequestStatus.NEW, c.getRequestStatus());
			Assert.assertEquals("test", c.getText());
		}

		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldcCreateCommentAtSolve.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldcCreateCommentAtSolve() {
		cz.tendersystems.helpdesk.entity.request.Request request = requestService.findRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");
		request.setSlaSchema(em.find(SlaSchema.class, 1l));
		request.setAuthor(em.find(Author.class, 1l));
		request.setComments(new ArrayList<>());

		commentService.createCommentAtSolve(request, prepareComment(), new ArrayList<>());

		List<CommentData> comments = commentService.loadCommentRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		for (CommentData c : comments) {
			Assert.assertEquals("zbytos@gmail.com", c.getAuthor());
			Assert.assertNotNull(c.getCreatedDateTime());
			Assert.assertEquals(RequestStatus.NEW, c.getRequestStatus());
			Assert.assertEquals("test", c.getText());
		}

		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldFailToCreateCommentOnSecurityException.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldFailToCreateCommentOnSecurityException() {
		try {
			commentService.createComment("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", new CommentData(), new ArrayList<>());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni vytvorit komentar", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/commentServiceTest/shouldCreateComment.xml")
	@ApplyScriptBefore("datasets/commentServiceTest/cleanup.sql")
	public void shouldCreateComment() {
		login(LOGIN_NAME_ADMINISTRATOR);

		commentService.createComment("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", prepareComment(), new ArrayList<>());

		List<CommentData> comments = commentService.loadCommentRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		for (CommentData c : comments) {
			Assert.assertEquals("zbytos@gmail.com", c.getAuthor());
			Assert.assertNotNull(c.getCreatedDateTime());
			Assert.assertEquals(RequestStatus.NEW, c.getRequestStatus());
			Assert.assertEquals("test", c.getText());
		}

		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	private CommentData prepareComment() {
		CommentData comment = new CommentData();
		comment.setAuthor("zbytos@gmail.com");
		comment.setRequestStatus(RequestStatus.NEW);
		comment.setText("test");

		return comment;
	}

}
