package cz.tendersystems.helpdesk.service.request;

import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.request.Request;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.validation.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.security.*;

@RunWith(Arquillian.class)
public class RequestServiceTest extends AbstractEJBTest {

	@EJB
	RequestService requestService;

	@EJB
	SlaSchemaService slaSchemaService;

	@EJB
	UserService userService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();

		jar.addClasses(RequestService.class, RequestSecurityService.class, UserService.class, SlaSchemaService.class, OutsourcerService.class, ProductService.class, VersionType.class);
		jar.addClasses(PasswordService.class, TimeActivityService.class, MailService.class, EmbedTemplate.class, MailTemplateService.class, ConfigurationService.class, IdService.class);
		jar.addClasses(CommentService.class, FileService.class, ValidationRequest.class, ToolsForRequestDates.class, ToolsForHolidays.class);

		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldFailToFindRequestOnSecurityException.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldFailToFindRequestOnSecurityException() {
		try {
			requestService.findRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist pozadavek", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldFindRequest.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldFindRequest() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Request request = requestService.findRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertNotNull(request);
		Assert.assertEquals("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", request.getId());
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldFailToFindRequestOnSecurityException.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldFailToLoadRequestOnSecurityException() {
		try {
			requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist pozadavek", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldFindRequest.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldLoadRequest() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestData request = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertNotNull(request);
		Assert.assertEquals("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", request.getId());
	}

	@Test
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldFailToCreateRequestOnValidationException() {
		try {
			requestService.createRequest(new RequestData(), null, null);
		} catch (ValidationException e) {
			Assert.assertNotNull(e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldCreateRequestAnonymous.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldCreateRequestAnonymous() throws ValidationException {
		RequestData data = new RequestData();
		data.setName("Problém");
		data.setSlaSchema(slaSchemaService.loadSlaSchema(1l));
		data.setAuthor(prepareAuthor(false));

		String id = requestService.createRequest(data, prepareComment(), new ArrayList<FileData>());

		Request request = em.find(Request.class, id);

		Assert.assertEquals("Problém", request.getName());
		Assert.assertEquals(RequestStatus.NEW, request.getRequestStatus());
		Assert.assertNotNull(request.getCreatedDateTime());
		Assert.assertNotNull(request.getReactionTime());
		Assert.assertNotNull(request.getTimeToSolve());
		Assert.assertNull(request.getDateOfClose());
		Assert.assertNotNull(request.getSlaSchema());
		Assert.assertNotNull(request.getAuthor());
		Assert.assertNull(request.getSolver());
		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldCreateRequestOutsourcer.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldCreateRequestOutsourcer() throws ValidationException {
		RequestData data = new RequestData();
		data.setName("Problém");
		data.setSlaSchema(slaSchemaService.loadSlaSchema(1l));
		data.setAuthor(prepareAuthor(true));

		String id = requestService.createRequest(data, prepareComment(), new ArrayList<FileData>());

		Request request = em.find(Request.class, id);

		Assert.assertEquals("Problém", request.getName());
		Assert.assertEquals(RequestStatus.NEW, request.getRequestStatus());
		Assert.assertNotNull(request.getCreatedDateTime());
		Assert.assertNotNull(request.getReactionTime());
		Assert.assertNotNull(request.getTimeToSolve());
		Assert.assertNull(request.getDateOfClose());
		Assert.assertNotNull(request.getSlaSchema());
		Assert.assertNotNull(request.getAuthor());
		Assert.assertNull(request.getSolver());
		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldFailToUpdateRequestOnSecurityException.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldFailToUpdateRequestOnSecurityException() throws ValidationException {
		try {
			requestService.updateRequest(requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"), new CommentData(), RequestStatus.ARCHIVED, new ArrayList<>());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist pozadavek", e.getMessage());
		}
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldUpdateRequestAnonymous.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldUpdateRequestAnonymous() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestData request = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		requestService.updateRequest(request, prepareComment(), RequestStatus.INHERITED, new ArrayList<>());

		RequestData updateRequest = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertEquals("test", updateRequest.getName());
		Assert.assertEquals(RequestStatus.INHERITED, updateRequest.getRequestStatus());
		Assert.assertNotNull(updateRequest.getCreatedDateTime());
		Assert.assertNotNull(updateRequest.getReactionTime());
		Assert.assertNotNull(updateRequest.getTimeToSolve());
		Assert.assertNull(updateRequest.getDateOfClose());
		Assert.assertNotNull(updateRequest.getSlaSchema());
		Assert.assertNotNull(updateRequest.getAuthor());
		Assert.assertNotNull(updateRequest.getSolver());
		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldUpdateRequestOutsourcer.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldUpdateRequestOutsourcer() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestData request = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		requestService.updateRequest(request, prepareComment(), RequestStatus.INHERITED, new ArrayList<>());

		RequestData updateRequest = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertEquals("test", updateRequest.getName());
		Assert.assertEquals(RequestStatus.INHERITED, updateRequest.getRequestStatus());
		Assert.assertNotNull(updateRequest.getCreatedDateTime());
		Assert.assertNotNull(updateRequest.getReactionTime());
		Assert.assertNotNull(updateRequest.getTimeToSolve());
		Assert.assertNull(updateRequest.getDateOfClose());
		Assert.assertNotNull(updateRequest.getSlaSchema());
		Assert.assertNotNull(updateRequest.getAuthor());
		Assert.assertNotNull(updateRequest.getSolver());
		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	@Test
	@UsingDataSet("datasets/requestServiceTest/shouldUpdateRequestStatusOnSolved.xml")
	@ApplyScriptBefore("datasets/requestServiceTest/cleanup.sql")
	public void shouldUpdateRequestStatusOnSolved() throws ValidationException {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestData request = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		requestService.updateRequest(request, prepareComment(), RequestStatus.SOLVED, new ArrayList<>());

		RequestData updateRequest = requestService.loadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0");

		Assert.assertEquals("test", updateRequest.getName());
		Assert.assertEquals(RequestStatus.SOLVED, updateRequest.getRequestStatus());
		Assert.assertNotNull(updateRequest.getCreatedDateTime());
		Assert.assertNotNull(updateRequest.getReactionTime());
		Assert.assertNotNull(updateRequest.getTimeToSolve());
		Assert.assertNotNull(updateRequest.getDateOfClose());
		Assert.assertNotNull(updateRequest.getSlaSchema());
		Assert.assertNotNull(updateRequest.getAuthor());
		Assert.assertNotNull(updateRequest.getSolver());
		Assert.assertTrue(!em.createQuery("FROM ApplicationLog").getResultList().isEmpty());
	}

	private AuthorData prepareAuthor(boolean loggedUser) {
		AuthorData author = new AuthorData();

		if (loggedUser) {
			author.setLoggedUser((UserOutsourcerData) userService.loadUser("uzivatelTa"));
		} else {
			author.setCompanyName("Tendersystems");
			author.setAnonymous(preparePerson());
		}

		return author;
	}

	private PersonData preparePerson() {
		PersonData person = new PersonData();
		person.setName("Zbyněk");
		person.setSurname("Richter");
		person.setMail("zbytos@gmail.com");

		return person;
	}

	private CommentData prepareComment() {
		CommentData comment = new CommentData();

		comment.setAuthor("Zbyněk Richter");
		comment.setText("Problém");
		comment.setRequestStatus(RequestStatus.NEW);

		return comment;
	}
}
