package cz.tendersystems.helpdesk.security;

import java.io.*;
import java.text.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.enterprise.context.*;
import javax.faces.context.*;
import javax.inject.*;
import javax.servlet.*;
import javax.servlet.http.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.general.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.general.jsf.parameter.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.security.*;

@Named
@RequestScoped
public class AuthenticationAction extends AbstractAction implements Serializable {

	private static final long serialVersionUID = 3729982123285353459L;

	private static final int intervalForExpiresSession = 1200000;
	private static final int intervalForAutomaticLogout = 1200000;

	private static final String administrationPage = "/administration/administrationOutsourcers.jsf";
	private static final String requestPage = "/request/requests.jsf";
	private static final String index = "/index.jsf";

	@EJB
	ApplicationLogService applicationLogService;

	@EJB
	BasicSecurityService basicSecurityService;

	@Inject
	@Parameter
	private String automatic;

	private String loginName;
	private String password;
	private String url;

	@PostConstruct
	public void init() {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		url = (String) externalContext.getRequestMap().get(RequestDispatcher.FORWARD_REQUEST_URI);

		if (url != null) {
			String original = (String) externalContext.getRequestMap().get(RequestDispatcher.FORWARD_QUERY_STRING);

			if (original != null) {
				url += "?" + original;
			}
		}
	}

	public void login() {
		String authenticationKey = MessageFormat.format("{0}{1}", password, loginName);
		FacesContext context = FacesContext.getCurrentInstance();
		ExternalContext externalContext = context.getExternalContext();
		HttpServletRequest httpServletRequest = (HttpServletRequest) externalContext.getRequest();

		try {
			if (httpServletRequest.getUserPrincipal() == null) {
				log.infov("Zpracovani prihlaseni pro prihlasovaci jmeno [{0}], heslo zadano [{1}]", loginName, password != null);

				httpServletRequest.login(loginName, authenticationKey);

				externalContext.redirect(prepareLinkForRedirect());

				createLog(new ApplicationLogData(LevelRecord.INFORMATION, "Přihlášení uživatele", null, MessageFormat.format("Přihlášení uživatele z IP adresy {0}", httpServletRequest.getRemoteAddr())));
				writeFacesMessage("information.successfulLoggedUser", LogLevel.INFORMATION);
			} else {
				writeFacesMessage("warning.userHadAlreadyLogged", LogLevel.WARNING, loginName);
			}
		} catch (ServletException e) {
			createErrorLog("Selhání přihlášení uživatel {0} z IP adresy {1} z důvodu špatných přihlašovacích údajů", httpServletRequest.getRemoteAddr());
			writeFacesMessage("error.badLoginInformation", LogLevel.ERROR);
			System.out.println(e);
		} catch (Exception e) {
			createErrorLog("Selhání přihlášení uživatele {0} z IP adresy {1} z důvodu neočekávané situace", httpServletRequest.getRemoteAddr());
			writeFacesMessage("error.unexpectedSituation", LogLevel.SEVERITY_FATAL, loginName);
		} finally {
			loginName = null;
		}
	}

	public void logout(boolean automatic) throws IOException {
		ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		HttpServletRequest httpServletRequest = (HttpServletRequest) externalContext.getRequest();

		loginName = httpServletRequest.getRemoteUser();

		try {
			httpServletRequest.logout();

			createLog(new ApplicationLogData(LevelRecord.INFORMATION, automatic ? "Automatické odhlášení uživatele" : "Odhlášení uživatele", null, null));
			writeFacesMessage(automatic ? "information.automaticLogout" : "information.successfullLogout", LogLevel.INFORMATION, (Object) loginName);
		} catch (Exception e) {
			writeFacesMessage("error.unexpectedSituation", LogLevel.SEVERITY_FATAL, loginName);
		} finally {
			externalContext.invalidateSession();
			externalContext.redirect(MessageFormat.format("{0}/logout.jsf?automatic={1}", externalContext.getRequestContextPath(), automatic));
		}
	}

	public String prepareLinkForRedirect() {
		if (url != null && !url.isEmpty()) {
			return url;
		} else if (basicSecurityService.isUserAdministrator()) {
			return prepareUrl(administrationPage);
		} else if (!basicSecurityService.isUserAdministrator() && basicSecurityService.isUserLogged()) {
			return prepareUrl(requestPage);
		}

		return prepareUrl(index);
	}

	private String prepareUrl(String page) {
		return FacesContext.getCurrentInstance().getExternalContext().getRequestContextPath() + page;
	}

	private void createLog(ApplicationLogData applicationLog) {
		applicationLog.setLoginName(loginName);

		applicationLogService.createLog(this.getClass(), applicationLog);
	}

	private void createErrorLog(String details, String ip) {
		ApplicationLogData applicationLog = new ApplicationLogData(LevelRecord.ERROR, "Selhání přihlášení uživatele", null, MessageFormat.format(details, loginName, ip));

		applicationLogService.createLog(this.getClass(), applicationLog);
	}

	public int getIntervalForExpiresSession() {
		return intervalForExpiresSession;
	}

	public int getIntervalForAutomaticLogout() {
		return intervalForAutomaticLogout;
	}

	public String getLoginName() {
		return loginName;
	}

	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getAutomatic() {
		return automatic;
	}
}